unit observer_core;

interface

uses
  Classes;

type
//  IPublisher = interface
//  ['{A7AE1AC5-6642-4DAB-AF22-57BABD78BA0F}']
//    function GetData (strWhat: string);
//  end;

  ISubscribe = interface
    ['{586A80B2-0357-4023-A7F1-549881CC2F0C}']
    procedure Notify (strWhat: string; data: TObject);
  end;


  IPubAndSubcr = interface
    ['{DD875933-73E6-46C2-A95D-3B8632A91C23}']
//    procedure Publish (pub: IPublisher; strWhat: string);
    procedure SimplePublish (strWhat: string; data: TObject);
    procedure Subscribe (subscriber: TObject; strWhat: string);
  end;

type
  TObserver = class (TInterfacedObject, IPubAndSubcr)
  private
    sList: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    class function Instance: TObserver;
    procedure SimplePublish(strWhat: string; data: TObject);
    procedure Subscribe(subscriber: TObject; strWhat: string);
  end;


implementation

uses
  SysUtils;

var
  _Obs: TObserver;

{ TObserver }

constructor TObserver.Create;
begin
  sList := TStringList.Create;
end;

procedure TObserver.SimplePublish(strWhat: string; data: TObject);
var
  I: Integer;
  intf: ISubscribe;
begin
  for I := 0 to sList.Count -1 do
  begin
    if SameText (strWhat, sList[I]) then
    begin
      sList.Objects[I].GetInterface(ISubscribe, intf);
      intf.Notify(strWhat, data);
    end;
  end;
end;

procedure TObserver.Subscribe(subscriber: TObject; strWhat: string);
begin
  if Supports (subscriber, ISubscribe) then
    sList.AddObject(strWhat, subscriber);
end;

destructor TObserver.Destroy;
begin
  sList.Free;
  inherited;
end;

class function TObserver.Instance: TObserver;
begin
  if not Assigned (_Obs) then
    _Obs := TObserver.Create;
  Result := _Obs;
end;

end.
